import config from "@/api/config.js";
import * as KMap from "@/utils/ol-map/KMap";
import * as util from "@/common/ol_common.js";
import Style from "ol/style/Style";
import Icon from "ol/style/Icon";
import Feature from "ol/Feature";
import VectorLayer from "ol/layer/Vector.js";
import WKT from "ol/format/WKT.js";
import { reactive } from "vue";
import { newPoint } from "@/utils/map.js";
import { remove } from "@/utils/ol-map/Polygon.js";
import { getArea } from "ol/sphere.js";
import * as proj from "ol/proj";
import proj4 from "proj4";
import { register } from "ol/proj/proj4";
proj4.defs(
  "EPSG:38572",
  "+proj=merc +a=6378137 +b=6378137 +lat_ts=0 +lon_0=0 +x_0=0 +y_0=0 +k=1 +units=m +nadgrids=@null +wktext +no_defs +type=crs"
);
register(proj4);
export let mapData = reactive({
  isEdit: false,
  isEditArea: false,
  curPointData: {},
  point: null,
});

/**
 * @description 地图层对象
 */
class AuthenticMap {
  constructor() {
    let that = this;
    let vectorStyle = new KMap.VectorStyle();
    this.vectorStyle = vectorStyle;
    // 位置图标
    this.clickPointLayer = new KMap.VectorLayer("clickPointLayer", 9999, {
      style: (f) => {
        return new Style({
          image: new Icon({
            src: require(`@/assets/images/common/${f.get("icon")}-icon.png`),
            scale: 0.45,
          }),
        });
      },
    });
  }

  initMap(location, target) {
    let level = 16;
    let coordinate = util.wktCastGeom(location).getFirstCoordinate();
    this.kmap = new KMap.Map(
      target,
      level,
      coordinate[0],
      coordinate[1],
      null,
      6,
      22
    );
    this.kmap.initDraw((e) => {
      mapData.isEdit = true;
      console.log('e',e);
      mapData.point = e.feature
    });
    this.kmap.startDraw();
    this.kmap.addLayer(this.clickPointLayer.layer);
    this.addMapSingerClick();
  }

  //   添加点位
  addPoint(points) {
    const arrPoints = [];
    if (points && points.length > 0) {
      points.forEach((item) => {
        arrPoints.push(newPoint({ ...item, icon: "point" }, "point"));
      });
      this.clickPointLayer.source.addFeatures(arrPoints);
    }
  }

  endEdit() {
    this.kmap.endDraw();
    this.kmap.modifyDraw();
    mapData.isEdit = false;
  }
  a() {
    // this.kmap.endDraw();
    this.kmap.startDraw();
  }

  // 地图点击事件
  addMapSingerClick() {
    let that = this;
    that.kmap.on("singleclick", (evt) => {
      that.kmap.map.forEachFeatureAtPixel(evt.pixel, function (feature, layer) {
        // 点击的图层是否是VectorLayer
        if (
          layer instanceof VectorLayer &&
          layer.get("name") === "clickPointLayer"
        ) {
          const icon = feature.get("icon") === "point" ? "point-act" : "point";
          feature.set("icon", icon);
          mapData.curPointData = {};
          mapData.point = feature;
          mapData.curPointData = feature.values_;
          console.log('this.kmap.getLayerFeatures()',that.kmap.getLayerFeatures());
          that.kmap.endDraw();
          that.kmap.modifyDraw((e) => {
            mapData.isEdit = false;
            mapData.isEditArea = false;
            mapData.isEditArea = true;
            
          });
        } else if (
          layer instanceof VectorLayer &&
          layer.get("name") === "defaultPolygonLayer"
        ) {
          that.kmap.endDraw();
          that.kmap.startDraw();
        } else {
          that.kmap.startDraw();
        }
      });
    });
  }

  setPoint(name) {
    mapData.point.set("icon", name);
  }

  //添加地块
  setAreaGeometry(geometryArr) {
    let that = this;
    geometryArr.map((item) => {
      that.kmap.setLayerWkt(item.featureWkt);
    });
  }

  b(){
    // console.log('this.kmap',this.kmap.polygonLayer)
    // console.log('point',mapData.point);
    // let that = this
    // this.clickPointLayer.source.forEachFeature((f) => {
    //     console.log('f',f);
    //     if (f.get("id") === mapData.point.get("id")) {
    //         that.kmap.polygonLayer.source.removeFeature(f);
    //         // that.kmap.remove(f)
    //     //   this.kmap.getView().setCenter(extractCoordinates(new WKT().writeFeature(f)));
    //     //   this.kmap.getView().setZoom(16);
    //     }
    //   })
    // this.kmap.removeLayer(this.kmap.polygonLayer.layer)
    // this.kmap.addLayer(this.kmap.polygonLayer.layer)
  }

  //获取地块信息
  getAreaGeometry() {
    const features = this.kmap.getLayerFeatures();
    let geometryArr = [];
    let area = 0;
    // 获取图层上的Polygon，转成geoJson用于回显
    features.forEach((item) => {
      geometryArr.push({ featureWkt: new WKT().writeFeature(item) });
      let geom = item.getGeometry().clone();
      geom.transform(proj.get("EPSG:4326"), proj.get("EPSG:38572"));
      let areaItem = getArea(geom);
      areaItem = (areaItem + areaItem / 2) / 1000;
      area += areaItem;
    });
    return { geometryArr, area: area.toFixed(2) };
  }
}

export default AuthenticMap;
